#	13 Apr 2020
#	example for LTS maximum likelihood estimation
#	Berenguer-Rico, Johansen, Nielsen (2021)
#	A model where the Least Trimmed Squares estimator is maximum likelihood
#
#	Uses State Infant Mortality from Wooldridge	(2015)
#	Data has 51 observations: 50 states plus Washington	DC

###################
#	Start by running code in ltsReg_find_h.r
###################

library(wooldridge)
library(plotrix)

###################
#	set path for graphics output

Drive	<- "c:/mydirectory/"

###################
#	Set random seed before each LTS call
do.set.seed	<- TRUE	

###################
#	Set up data
#	We only use data for the year 1990
data.set	<- infmrt[	which(infmrt$year=='1990'), ]	# Wooldridge

#	log transform dependent variable

linfmort	<- log(data.set$infmort)

#	Add information about states
#	Taken from US Bureau of the Census, 1994, Table 123, page 92
#	https://www.census.gov/library/publications/1994/compendia/statab/114ed.html
#	http://www2.census.gov/library/publications/1994/compendia/statab/114ed/1994-02.pdf

states		<- c(	"ME","NH","VT","MA","RI", "CT","NY","NJ","PA","OH",
					"IN","IL","MI","WI","MN", "IA","MO","ND","SD","NE",
					"KS","DE","MD","DC","VA", "WV","NC","SC","GA","FL",
					"KY","TN","AL","MS","AR", "LA","OK","TX","MT","ID",
					"WY","CO","NM","AZ","UT", "NV","WA","OR","CA","AK",
					"HI"												)

#	Add information about black population (in 1000s):
#	Taken from US Bureau of the Census, 1992, Table 255
#	1990 Census of Population: General Population Characteristics, CP-1-1, page 364, table 255
#	https://www.census.gov/library/publications/1992/dec/cp-1.html

popul.black	<- c(	   5.1,   7.2,   2.0, 300.1,  38.9,  274.3,2859.1,1037.8,1089.8,1154.8,
					 432.1,1694.3,1291.7, 244.5,  94.9,   48.1, 548.2,   3.5,   3.3,  57.4,
					 143.1,	112.5,1189.9, 399.6,1163.0,   56.3,1456.3,1039.9,1746.6,1759.5,
					 262.9, 778.0,1020.7, 915.1, 373.9, 1299.3, 233.8,2021.6,   2.4,   3.4,
					   3.6, 133.1,  30.2, 110.5,  11.6,   78.8, 149.8,  46.2,2208.8,  22.5,
					  27.2																	) 
lblack		<- log(popul.black) - data.set$lpopul

#	Take logs of afdc variable

lafdc		<- log(data.set$afdcper/100)

#	Include in dataset
data.set	<- cbind(data.set,linfmort,states,lblack,lafdc)

##############################
#	Standard LTS analysis
if(do.set.seed)	set.seed(0)
fit.LTS.1	<- ltsReg( infmort ~ lpcinc + lphysic + lpopul					,data.set)
if(do.set.seed)	set.seed(0)
fit.LTS.2	<- ltsReg(linfmort ~ lpcinc + lphysic + lpopul + lblack			,data.set)
if(do.set.seed)	set.seed(0)
fit.LTS.3	<- ltsReg(linfmort ~ lpcinc + lphysic + lpopul + lblack + lafdc	,data.set)
if(do.set.seed)	set.seed(0)
fit.LTS.4	<- ltsReg(linfmort ~ 	 	  lphysic 		   + lblack 		,data.set)

#	Index plots
ltsPlot(fit.LTS.1,which="rindex")	#	outliers: 24
ltsPlot(fit.LTS.2,which="rindex")	#	outliers: 19,24,39
ltsPlot(fit.LTS.3,which="rindex")	#	outliers: 19,24,50
ltsPlot(fit.LTS.4,which="rindex")	#	outliers: 24

states[c(19,24,39,50)]				#	"SD" "DC" "MT" "AK"

###############################
# 	Estimate h in LTS model
#
#	Regressand
Y	<- data.set$infmort
#	Regressors 
X.1 <- cbind( data.set$lpcinc , data.set$lphysic , data.set$lpopul )
X.2 <- cbind( data.set$lpcinc , data.set$lphysic , data.set$lpopul , data.set$lblack )
X.3 <- cbind( data.set$lpcinc , data.set$lphysic , data.set$lpopul , data.set$lblack ,	data.set$lafdc )
X.4 <- cbind( 					data.set$lphysic , 					 data.set$lblack )
colnames(X.1) <- c("lpcinc","lphysic","lpopul")
colnames(X.2) <- c("lpcinc","lphysic","lpopul","lblack")
colnames(X.3) <- c("lpcinc","lphysic","lpopul","lblack","lafdc")
colnames(X.4) <- c(			"lphysic",		   "lblack")

#	Estimating h
tab.LTS.1	<- ltsReg_find_h(	 Y , X.1 ,do.set.seed=do.set.seed)	
tab.LTS.2	<- ltsReg_find_h(log(Y), X.2 ,do.set.seed=do.set.seed)
tab.LTS.3	<- ltsReg_find_h(log(Y), X.3 ,do.set.seed=do.set.seed)
tab.LTS.4	<- ltsReg_find_h(log(Y), X.4 ,do.set.seed=do.set.seed)

###############################
#	Report preferred LTS estimates
#	In LTS, can use standard OLS inference for good observations

#	model 1, h = 45
if(do.set.seed)	set.seed(0)
fit.LTS.h.1	<- ltsReg( infmort ~ lpcinc + lphysic + lpopul					,data.set,alpha=alpha.h.n(45,51,4))
fit.lm.h.1	<-     lm( infmort ~ lpcinc + lphysic + lpopul					,data.set[fit.LTS.h.1$best,])
summary(fit.lm.h.1)
#	model 2, h = 50
if(do.set.seed)	set.seed(0)
fit.LTS.h.2	<- ltsReg(linfmort ~ lpcinc + lphysic + lpopul + lblack			,data.set,alpha=alpha.h.n(50,51,5))
fit.lm.h.2	<-     lm(linfmort ~ lpcinc + lphysic + lpopul + lblack			,data.set[fit.LTS.h.2$best,])
summary(fit.lm.h.2)
#	model 3, h = 50
if(do.set.seed)	set.seed(0)
fit.LTS.h.3	<- ltsReg(linfmort ~ lpcinc + lphysic + lpopul + lblack + lafdc	,data.set,alpha=alpha.h.n(50,51,6))
fit.lm.h.3	<-     lm(linfmort ~ lpcinc + lphysic + lpopul + lblack	+ lafdc	,data.set[fit.LTS.h.3$best,])
summary(fit.lm.h.3)


###############################
#	Figure 8.3 in paper

Width	<- 8
Height	<- 4
filename	<- "Fig3"
postscript(file=paste(Drive,filename,".eps",sep=""),horizontal=FALSE,family="Times",onefile=FALSE,bg="transparent",fg="black",paper="special",width=Width,height=Height)

from	<- 5.1
to		<- 35.2
ytics	<- c(0:4,36,37)
yticlab <- c(0:4,36,37)

par(cex.axis=1.4,cex.lab=1.4)
#par(mar=c(4.5, 5, 0.5, 0.5))		if labels on axis
par(mar=c(2.5, 3, 0.5, 0.5))
gap.plot(tab.LTS.1[,2],tab.LTS.1[,12], gap=c(from,to), type="b", xlab="h", ylab="T criterion",ylim=c(0,37),ytics=ytics,yticlab=yticlab)
axis.break(2, from, breakcol="snow", style="gap")
axis.break(2, from*(1+0.02), breakcol="black", style="slash")
axis.break(4, from*(1+0.02), breakcol="black", style="slash")
lines(c(50,51),c(0.9546,6.58), type="b",lty=2)	# auxiliary line accross panels.						
lines(tab.LTS.2[,2],tab.LTS.2[,13],type="b",pch=22,col="blue")
#legend("topleft",legend=c("  infmrt on lpcinc,lphysic,lpopul"		,
#						 "linfmrt on lpcinc,lphysic,lpopul,lblack"	),
#				 pch=c(21,22),
#				 col=c("black","red"))

dev.off()
