#	13 Apr 2020
#	example for LTS maximum likelihood estimation
#	Berenguer-Rico, Johansen, Nielsen (2021)
#	A model where the Least Trimmed Squares estimator is maximum likelihood
#
#	Uses State Infant Mortality from Wooldridge	(2015)
#	Data has 51 observations: 50 states plus Washington	DC

###################
#	Start by running code in ltsReg_find_h.r
###################

library(robustbase)

###################
#	set path for graphics output

Drive	<- "c:/mydirectory/"

###################
#	Set random seed before each LTS call
do.set.seed	<- TRUE

######################
data.set	<- starsCYG	# robustbase
Y			<- data.set$log.light
X			<- data.set$log.Te
Y.name		<- "log light intensity"
X.name		<- "log temperature"

p			<- 2			

##############################
#	Standard LTS analysis
if(do.set.seed)	set.seed(0)
lts	<- ltsReg(Y~X, data = data.set)

#	Index plots
ltsPlot(lts,which="rindex")	#	outliers: 7,11,20,30,34



###############################
#	Figure 8.1 in paper

Width	<- 8
Height	<- 4
filename	<- "Fig1"
postscript(file=paste(Drive,filename,".eps",sep=""),horizontal=FALSE,family="Times",onefile=FALSE,bg="transparent",fg="black",paper="special",width=Width,height=Height)

par(cex.axis=1.4,cex.lab=1.4)
#par(mar=c(4.5, 5, 0.5, 0.5))		if labels on axis
par(mar=c(2.5, 2.4, 0.5, 0.5))

h	<- c(25,42,43,44,47)
plot(X,Y,xlab=X.name,ylab=Y.name,xlim=rev(range(X)),ylim=c(min(Y)-0.05,max(Y)))
col	 <- c(1,1,1,1,1)
lty	 <- c(5,1,5,5,5)
for(i in 1:length(h))
{
	if(do.set.seed)	set.seed(0)
	lts	<- ltsReg(Y~X, data = data.set,alpha=alpha.h.n(h[i],n,p))
	lines(rev(range(X)),lts$raw.coefficients[1]+lts$raw.coefficients[2]*rev(range(X)),col=col[i],lty=lty[i])
}
h0	<- 42
lts	<- ltsReg(Y~X, data = data.set,alpha=alpha.h.n(h0,n,p))
points(X[lts$best],Y[lts$best],col=col[2],pch=19)
text(4.095,3.90,"h=25",cex=1.4)
text(3.965,3.90,"h=42",cex=1.4)
text(3.81 ,3.90,"h=43",cex=1.4)
text(3.49 ,4.28,"h=44",cex=1.4)
text(3.49 ,5.29 ,"h=47",cex=1.4)

dev.off()

###############################
#	Table 8.1 in paper

if(do.set.seed)	set.seed(0)
tab.LTS	<- ltsReg_find_h(Y,X)	
tab.LTS[c(1,12,13,16:23),c(2,3,4,6,10)]

###############################
#	Table 8.2 in paper

index.obs	<- setdiff(1:length(Y) , 7)
if(do.set.seed)	set.seed(0)
tab.LTS	<- ltsReg_find_h(Y[index.obs],X[index.obs])	
tab.LTS[c(18:23),c(2,3,4,6,10)]

###############################
#	Figure 8.2 in paper

Width	<- 8
Height	<- 8/3
filename	<- "Fig2"
postscript(file=paste(Drive,filename,".eps",sep=""),horizontal=FALSE,family="Times",onefile=FALSE,bg="transparent",fg="black",paper="special",width=Width,height=Height)

h	<- c(25,42,47)
n	<- length(Y)
par(mfrow=c(1,3))
par(mar=c(0.5,0,0,0)+0.1)
par(cex.axis=1.6,cex.lab=1.6,cex.main=1.6)
for(i in 1:length(h))
{
	if(do.set.seed)	set.seed(0)
	lts	<- ltsReg(Y~X, data = data.set,alpha=alpha.h.n(h[i],n,p))
	if(i==length(h))	lts$best	<- 1:n
	residuals <- Y - lts$raw.coefficients[1] - lts$raw.coefficients[2]*X
	good.residuals	<- residuals[lts$best]
	RSS <- sum(good.residuals^2)
	sigma	<- sqrt(RSS/h[i])
	main <- paste("LTS with h=",h[i])
	bw	<- 1.5*(h[i]^{-0.2})	 # very sensitive to this choice. 1.5*(h^{-0.2}) gives nice fit for h=42
	par(mar=c(2.5,2.5,2,0.5))
	plot(density(residuals/sigma,bw=bw),ylim=c(0,0.4),main=main)
	x <- seq(-4,4,by=0.1)
	lines(x,dnorm(x)*h[i]/n,col="blue",lwd=2)
	dens <- density(good.residuals/sigma,bw=bw)
	lines(dens$x,dens$y*h[i]/n,col="red",lty=2,lwd=2)
}

dev.off()





