#	13 Apr 2020
#	functions for LTS maximum likelihood estimation
#	Berenguer-Rico, Johansen, Nielsen (2021)
#	A model where the Least Trimmed Squares estimator is maximum likelihood
#
#	Code for estimating h based on lts.Reg in robustbase
#
#	input	Y	vector of dependent variable
#  			X 	matrix of independent variable not including intercept,
#				at least one regressor	

library(robustbase)

###########################################
#	function to find alpha from h
#	inverse of h.alpha.n in robustbase
alpha.h.n <- function(h,n,p,epsilon=0.00001)
{
	if((n+p+1) %% 2 == 0) 	{ alpha <- ((h-p-1)/(n-p-1)+epsilon) }
	else				  	{ alpha <- ((h-p)/(n-p)+epsilon) }
	if(sum(abs(h.alpha.n(alpha,n,p)-h))!=0)	print("ok")
	if(alpha>1)	alpha <- 1
	return(alpha)
}

###########################################
#	function to estimate h.
#	uses information criteria
#	& cumulant bases criteria
#	input	Y			vector of dependent variable
#  			X 			matrix of independent variable not including intercept,
#						at least one regressor
#			do.set.seed	TRUE: set.seed(0) before each LTS call
#						FALSE: do not set seed.
#	output	table	
#	NOTE: it is important to use raw.coefficients,
#	which are the OLS estimates on the selected sample.
ltsReg_find_h	<- function (Y,X,do.set.seed=TRUE)
{	# 	ltsReg_find_h	
	###############
	#	find n,p
	n	<- length(Y)
	p	<- ncol(X)
	if(is.null(p))	p	<- 1
	p	<- p+1		# count intercept
	###############
	#	h range
	h	<- floor((n+p+1)/2):n
#	h	<- floor((n+p+1)/2)
#	ceiling(n/2+p):n
	###############
	#	set up table
	names	<- list(h,c("alpha","h",as.character(1:p),
					"RSS","sigma","cum3","cum4","|cum4|","T","IC",
					"sigma.df","cum3.df","cum4.df","|cum4|.df","T.df"))
	m.results	<- matrix(nrow=length(h),ncol=length(names[[2]]),dimnames=names)
	###############
	#	loop over h
	#	compute LTS using ltsReg
	#	compute derived statistics for each h
	for(i in 1:length(h))
	{
		alpha	<- alpha.h.n(h[i],n,p)
		if(do.set.seed)	set.seed(0)
		lts <- ltsReg(Y~X,alpha=alpha)
		if(p>2)
			{	residuals 	<- Y- lts$raw.coefficients[1] - X %*% lts$raw.coefficients[2:p]	}
		else
			{	residuals 	<- Y- lts$raw.coefficients[1] - X  *  lts$raw.coefficients[2]			}
		if(h[i]==n)	lts$best	<- 1:n		# ltsReg returns null	
		good.residuals	<- residuals[lts$best]
		RSS <- sum(good.residuals^2)
		#	statistics normalised by sample size h
		sigma2	<- RSS/h[i]					
		k3	<- h[i]*sum(good.residuals^3)/RSS^(3/2)/sqrt(6)
		k4	<- sqrt(h[i])*(h[i]*sum(good.residuals^4)/RSS^2 - 3)/sqrt(24)	
		Thiele	<- k3^2+k4^2
		#	statistics normalised by degrees of freedom h-p not counting intercept
		sigma2.df	<- RSS/(h[i]-p)
		k3.df	<- (h[i]-p)*sum(good.residuals^3)/RSS^(3/2)/sqrt(6)
		k4.df	<- sqrt(h[i]-p)*((h[i]-p)*sum(good.residuals^4)/RSS^2 - 3)/sqrt(24)	
		Thiele.df	<- k3.df^2+k4.df^2
		#	information criteria
		IC	<- log(sigma2)+2*log(log(h[i]))*(1-h[i]/n)
		m.results[i,]	<- c(alpha,lts$quan,lts$raw.coefficients,
								RSS,sqrt(sigma2),k3,k4,abs(k4),Thiele,IC,
								sqrt(sigma2.df),k3.df,k4.df,abs(k4.df),Thiele.df)
	}
	round(m.results,4)
}	# 	ltsReg_find_h

